@@ -26,6 +26,7 @@ import shutil
 import subprocess
 import sys
 import tempfile
+from urlparse import urlparse
 import webbrowser
 
 from solfege import mpd
@@ -47,7 +48,7 @@ def check_rcfile():
     """See default.config for rcfileversion values, meanings and
     a description of how to add config variables.
     """
-    rcfileversion = 19
+    rcfileversion = 20
     if cfg.get_int("app/rcfileversion") > rcfileversion:
         cfg.drop_user_config()
         return
@@ -128,6 +129,9 @@ def check_rcfile():
         for ex in ('singinterval', 'melodicinterval'):
             if cfg.get_int("%s/maximum_number_of_intervals" % ex) == 10:
                 cfg.set_int("%s/maximum_number_of_intervals" % ex, 12)
+    if cfg.get_int("app/rcfileversion") < 20:
+        cfg.del_key("gui/reserved_vspace")
+
     cfg.set_int("app/rcfileversion", rcfileversion)
     try:
         a = mpd.notename_to_int(cfg.get_string("user/lowest_pitch"))
@@ -182,7 +186,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 # We can get here if winmidi.output_devices() in winsynth
                 # __init__ returns no devices. Don't know when, but it could
                 # happen.
-                solfege.win.display_exception_message(e)
+                gu.display_exception_message(e)
                 cfg.set_string("sound/type", "fake-synth")
                 soundcard.initialise_using_fake_synth(True)
                 return
@@ -298,7 +302,7 @@ class SolfegeApp(cfg.ConfigUtils):
         except (lessonfile.LessonfileParseException,
                 dataparser.DataparserException), e:
             cleanup()
-            solfege.win.display_exception_message(e, lessonfile=filename)
+            gu.display_exception_message(e, lessonfile=filename)
             return
         if 'm_discards' in dir(self.m_teachers[module].m_P):
             for msg in self.m_teachers[module].m_P.m_discards:
@@ -314,17 +318,16 @@ class SolfegeApp(cfg.ConfigUtils):
     def test_lessonfile(self, filename):
         self.m_test_mode = True
         module = self.practise_lessonfile(filename)
-        solfege.win.box_dict[module].on_start_practise()
         solfege.win.enter_test_mode()
-    def handle_href(self, href, display_docfile_set_adj=1):
-        urlobj = utils.Url(href)
-        if not urlobj.protocol:
-            solfege.win.display_docfile(urlobj.filename, urlobj.anchor)
-        else:
+    def handle_href(self, href):
+        u = urlparse(href)
+        if u.scheme:
             try:
                 webbrowser.open_new(href)
             except Exception, e:
                 solfege.win.display_error_message2(_("Error opening web browser"), str(e))
+        else:
+            solfege.win.display_docfile(u.path)
     def create_teacher(self, modulename):
         """
         Create the teacher in 'modulename' and add it to self.m_teachers.
@@ -343,7 +346,7 @@ class SolfegeApp(cfg.ConfigUtils):
         try:
             cfg.sync()
         except IOError, e:
-            solfege.win.display_exception_message(e)
+            gu.display_exception_message(e)
         if isinstance(solfege.win.get_view(), abstract.Gui):
             g = solfege.win.get_view()
             # Check that the lesson file has a header, because if the
@@ -352,11 +355,13 @@ class SolfegeApp(cfg.ConfigUtils):
             # exceptions.
             if g.m_t.m_P and getattr(g.m_t.m_P, 'header', None):
                 g.on_end_practise()
+        solfege.db.conn.commit()
         solfege.db.conn.close()
         if soundcard.synth:
             soundcard.synth.close()
         shutil.rmtree(lessonfile.MusicBaseClass.temp_dir, True)
-    def export_training_set(self, export_data, export_dir, output_format):
+    def export_training_set(self, export_data, export_dir, output_format,
+                            name_track_by_question):
         """
         This function requires a program that can create WAV files
         from MIDI files and MP3 files from WAV.
@@ -386,6 +391,7 @@ class SolfegeApp(cfg.ConfigUtils):
             p = self.m_teachers[module].lessonfileclass()
             p.parse_file(lessonfile.uri_expand(filename))
             for c in range(lesson_info['count']):
+                trackname = "track-%i"
                 if module == 'idbyname':
                     p.select_random_question()
                     if p.header.lesson_heading:
@@ -395,8 +401,10 @@ class SolfegeApp(cfg.ConfigUtils):
                     table.append_row("%i" % track_idx,
                                      p.get_question().name,
                                      s)
+                    if name_track_by_question:
+                        trackname = "%s-%%i" % p.get_name()
                     soundcard.start_export(os.path.join(
-                            export_dir, "track-%i.mid" % track_idx))
+                            export_dir, "%s.mid" % trackname % track_idx))
                     for n in range(lesson_info.get('repeat', 1)):
                         p.play_question()
                         if n != lesson_info.get('repeat', 1) - 1:
@@ -411,10 +419,14 @@ class SolfegeApp(cfg.ConfigUtils):
                     t.q_status = t.QSTATUS_SOLVED
                     try:
                         table.append_row("%i" % track_idx, "%s" % utils.int_to_intervalname(t.m_interval))
+                        if name_track_by_question:
+                            trackname = "%%i-%s.mid" % utils.int_to_intervalname(t.m_interval)
                     except AttributeError:
                         table.append_row("%i" % track_idx, "%s" % (" + ".join([utils.int_to_intervalname(q, False, True) for q in t.m_question])))
+                        if name_track_by_question:
+                            trackname = "%%i-%s.mid" % ("+".join([utils.int_to_intervalname(q, False, True) for q in t.m_question]))
                     soundcard.start_export(os.path.join(
-                            export_dir, "track-%i.mid" % track_idx))
+                            export_dir, "%s.mid" % trackname % track_idx))
                     for n in range(lesson_info.get('repeat', 1)):
                         t.play_question()
                         if n != lesson_info.get('repeat', 1) - 1:
@@ -441,9 +453,9 @@ class SolfegeApp(cfg.ConfigUtils):
                     try:
                         inout = {
                             'in': os.path.join(export_dir,
-                                    "track-%i.%s" % (track_idx, from_ext)),
+                                    "%s.%s" % (trackname % track_idx, from_ext)),
                             'out': os.path.join(export_dir,
-                                    "track-%i.%s" % (track_idx, to_ext))}
+                                    "%s.%s" % (trackname % track_idx, to_ext))}
                         opts = cfg.get_string(app_cfg_name + '_options').split(" ")
                         opts = [x % inout for x in opts]
                         # For some reasong setting the executable arg does
@@ -455,8 +467,8 @@ class SolfegeApp(cfg.ConfigUtils):
                             raise osutils.BinaryForMediaConvertorException(app_cfg_name,
                                 cfg.get_string(app_cfg_name), e)
 
-                        if os.path.exists(os.path.join(export_dir, "track-%i.%s" % (track_idx, to_ext))):
-                            os.remove(os.path.join(export_dir, "track-%i.%s" % (track_idx, from_ext)))
+                        if os.path.exists(os.path.join(export_dir, "%s.%s" % (trackname % track_idx, to_ext))):
+                            os.remove(os.path.join(export_dir, "%s.%s" % (trackname % track_idx, from_ext)))
                         else:
                             # This means that the program failed to generate
                             # the WAV file. We set output_format to 'midi'
